@@ -1,48 +1,73 @@
-use std::sync::{
-    atomic::{AtomicBool, AtomicUsize, Ordering as AtomicOrdering},
-    Arc, RwLock,
+use std::{
+    collections::VecDeque,
+    sync::{
+        atomic::{AtomicBool, AtomicUsize, Ordering as AtomicOrdering},
+        Arc, RwLock,
+    },
 };
 
+#[cfg(all(target_arch = "wasm32", not(target_os = "wasi")))]
+use wasm_bindgen_test::wasm_bindgen_test;
+#[cfg(all(
+    all(target_arch = "wasm32", not(target_os = "wasi")),
+    feature = "browser",
+))]
+use wasm_bindgen_test::wasm_bindgen_test_configure;
+
 use callbag::{from_iter, Message};
 
-/// https://github.com/staltz/callbag-from-iter/blob/a5942d3a23da500b771d2078f296df2e41235b3a/test.js#L4-L34
+#[cfg(all(
+    all(target_arch = "wasm32", not(target_os = "wasi")),
+    feature = "browser",
+))]
+wasm_bindgen_test_configure!(run_in_browser);
+
+/// See <https://github.com/staltz/callbag-from-iter/blob/a5942d3a23da500b771d2078f296df2e41235b3a/test.js#L4-L34>
 #[test]
-fn it_sends_array_items_iterator_to_a_puller_sink() {
+#[cfg_attr(
+    all(target_arch = "wasm32", not(target_os = "wasi")),
+    wasm_bindgen_test
+)]
+fn it_sends_array_items_iterable_to_a_puller_sink() {
     let source = from_iter([10, 20, 30]);
 
-    let downwards_expected_types: Vec<(fn(&Message<_>) -> bool, &str)> = vec![
+    let downwards_expected_types: Vec<(fn(&Message<_, _>) -> bool, &str)> = vec![
         (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
-        (|m| matches!(m, Message::End), "Message::End"),
+        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
     ];
-    let downwards_expected_types = Arc::new(RwLock::new(downwards_expected_types.into_iter()));
+    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
+        Arc::new(RwLock::new(downwards_expected_types.into()));
     let downwards_expected = [10, 20, 30];
-    let downwards_expected = Arc::new(RwLock::new(downwards_expected.into_iter()));
+    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
+        Arc::new(RwLock::new(downwards_expected.into()));
 
     let talkback = Arc::new(RwLock::new(None));
     source(Message::Handshake(
         (move |message| {
             {
-                let mut downwards_expected_types = downwards_expected_types.write().unwrap();
-                let et = downwards_expected_types.next().unwrap();
+                let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
+                let et = downwards_expected_types.pop_front().unwrap();
                 assert!(et.0(&message), "downwards type is expected: {}", et.1);
             }
 
-            if let Message::Handshake(sink) = message {
+            if let Message::Handshake(source) = message {
                 {
                     let mut talkback = talkback.write().unwrap();
-                    *talkback = Some(sink);
+                    *talkback = Some(source);
                 }
                 let talkback = talkback.read().unwrap();
                 let talkback = talkback.as_ref().unwrap();
                 talkback(Message::Pull);
                 return;
             } else if let Message::Data(data) = message {
-                let mut downwards_expected = downwards_expected.write().unwrap();
-                let e = downwards_expected.next().unwrap();
-                assert_eq!(data, e, "downwards data is expected: {}", e);
+                {
+                    let downwards_expected = &mut *downwards_expected.write().unwrap();
+                    let e = downwards_expected.pop_front().unwrap();
+                    assert_eq!(data, e, "downwards data is expected: {}", e);
+                }
                 let talkback = talkback.read().unwrap();
                 let talkback = talkback.as_ref().unwrap();
                 talkback(Message::Pull);
@@ -52,8 +77,67 @@ fn it_sends_array_items_iterator_to_a_puller_sink() {
     ));
 }
 
-/// https://github.com/staltz/callbag-from-iter/blob/a5942d3a23da500b771d2078f296df2e41235b3a/test.js#L68-L97
+/// See <https://github.com/staltz/callbag-from-iter/blob/a5942d3a23da500b771d2078f296df2e41235b3a/test.js#L36-L66>
 #[test]
+#[cfg_attr(
+    all(target_arch = "wasm32", not(target_os = "wasi")),
+    wasm_bindgen_test
+)]
+fn it_sends_array_entries_iterator_to_a_puller_sink() {
+    let source = from_iter(["a", "b", "c"].into_iter().enumerate());
+
+    let downwards_expected_types: Vec<(fn(&Message<_, _>) -> bool, &str)> = vec![
+        (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
+        (|m| matches!(m, Message::Data(_)), "Message::Data"),
+        (|m| matches!(m, Message::Data(_)), "Message::Data"),
+        (|m| matches!(m, Message::Data(_)), "Message::Data"),
+        (|m| matches!(m, Message::Terminate), "Message::Terminate"),
+    ];
+    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
+        Arc::new(RwLock::new(downwards_expected_types.into()));
+    let downwards_expected = [(0, "a"), (1, "b"), (2, "c")];
+    let downwards_expected: Arc<RwLock<VecDeque<_>>> =
+        Arc::new(RwLock::new(downwards_expected.into()));
+
+    let talkback = Arc::new(RwLock::new(None));
+    source(Message::Handshake(
+        (move |message| {
+            {
+                let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
+                let et = downwards_expected_types.pop_front().unwrap();
+                assert!(et.0(&message), "downwards type is expected: {}", et.1);
+            }
+
+            if let Message::Handshake(source) = message {
+                {
+                    let mut talkback = talkback.write().unwrap();
+                    *talkback = Some(source);
+                }
+                let talkback = talkback.read().unwrap();
+                let talkback = talkback.as_ref().unwrap();
+                talkback(Message::Pull);
+                return;
+            } else if let Message::Data(data) = message {
+                {
+                    let downwards_expected = &mut *downwards_expected.write().unwrap();
+                    let e = downwards_expected.pop_front().unwrap();
+                    assert_eq!(data, e, "downwards data is expected: {:?}", e);
+                }
+                let talkback = talkback.read().unwrap();
+                let talkback = talkback.as_ref().unwrap();
+                talkback(Message::Pull);
+            }
+        })
+        .into(),
+    ));
+}
+
+/// See <https://github.com/staltz/callbag-from-iter/blob/a5942d3a23da500b771d2078f296df2e41235b3a/test.js#L68-L97>
+#[test]
+#[cfg_attr(
+    all(target_arch = "wasm32", not(target_os = "wasi")),
+    wasm_bindgen_test
+)]
 fn it_does_not_blow_up_the_stack_when_iterating_something_huge() {
     #[derive(Clone)]
     struct Gen {
@@ -90,10 +174,10 @@ fn it_does_not_blow_up_the_stack_when_iterating_something_huge() {
         ({
             let iterated = iterated.clone();
             move |message| {
-                if let Message::Handshake(sink) = message {
+                if let Message::Handshake(source) = message {
                     {
                         let mut talkback = talkback.write().unwrap();
-                        *talkback = Some(sink);
+                        *talkback = Some(source);
                     }
                     let talkback = talkback.read().unwrap();
                     let talkback = talkback.as_ref().unwrap();
@@ -104,7 +188,7 @@ fn it_does_not_blow_up_the_stack_when_iterating_something_huge() {
                     let talkback = talkback.as_ref().unwrap();
                     talkback(Message::Pull);
                     return;
-                } else if let Message::End = message {
+                } else if let Message::Terminate = message {
                     assert_eq!(
                         i.load(AtomicOrdering::Acquire),
                         1_000_000,
@@ -124,17 +208,22 @@ fn it_does_not_blow_up_the_stack_when_iterating_something_huge() {
     );
 }
 
-/// https://github.com/staltz/callbag-from-iter/blob/a5942d3a23da500b771d2078f296df2e41235b3a/test.js#L99-L129
+/// See <https://github.com/staltz/callbag-from-iter/blob/a5942d3a23da500b771d2078f296df2e41235b3a/test.js#L99-L129>
 #[test]
+#[cfg_attr(
+    all(target_arch = "wasm32", not(target_os = "wasi")),
+    wasm_bindgen_test
+)]
 fn it_stops_sending_after_source_completion() {
     let source = from_iter([10, 20, 30]);
 
     let actual = Arc::new(RwLock::new(vec![]));
-    let downwards_expected_types: Vec<(fn(&Message<_>) -> bool, &str)> = vec![
+    let downwards_expected_types: Vec<(fn(&Message<_, _>) -> bool, &str)> = vec![
         (|m| matches!(m, Message::Handshake(_)), "Message::Handshake"),
         (|m| matches!(m, Message::Data(_)), "Message::Data"),
     ];
-    let downwards_expected_types = Arc::new(RwLock::new(downwards_expected_types.into_iter()));
+    let downwards_expected_types: Arc<RwLock<VecDeque<_>>> =
+        Arc::new(RwLock::new(downwards_expected_types.into()));
 
     let talkback = Arc::new(RwLock::new(None));
     source(Message::Handshake(
@@ -142,15 +231,15 @@ fn it_stops_sending_after_source_completion() {
             let actual = actual.clone();
             move |message| {
                 {
-                    let mut downwards_expected_types = downwards_expected_types.write().unwrap();
-                    let et = downwards_expected_types.next().unwrap();
+                    let downwards_expected_types = &mut *downwards_expected_types.write().unwrap();
+                    let et = downwards_expected_types.pop_front().unwrap();
                     assert!(et.0(&message), "downwards type is expected: {}", et.1);
                 }
 
-                if let Message::Handshake(sink) = message {
+                if let Message::Handshake(source) = message {
                     {
                         let mut talkback = talkback.write().unwrap();
-                        *talkback = Some(sink);
+                        *talkback = Some(source);
                     }
                     let talkback = talkback.read().unwrap();
                     let talkback = talkback.as_ref().unwrap();
@@ -158,12 +247,12 @@ fn it_stops_sending_after_source_completion() {
                     return;
                 } else if let Message::Data(data) = message {
                     {
-                        let mut actual = actual.write().unwrap();
+                        let actual = &mut *actual.write().unwrap();
                         actual.push(data);
                     }
                     let talkback = talkback.read().unwrap();
                     let talkback = talkback.as_ref().unwrap();
-                    talkback(Message::End);
+                    talkback(Message::Terminate);
                     talkback(Message::Pull);
                     talkback(Message::Pull);
                 }
@@ -172,5 +261,5 @@ fn it_stops_sending_after_source_completion() {
         .into(),
     ));
 
-    assert_eq!(&actual.read().unwrap()[..], [10]);
+    assert_eq!((&*actual.read().unwrap())[..], [10]);
 }
